"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteEnvironmentParametersForResources = exports.deleteEnvironmentParametersFromService = void 0;
const amplify_cli_core_1 = require("@aws-amplify/amplify-cli-core");
const amplify_prompts_1 = require("@aws-amplify/amplify-prompts");
const aws_ssm_1 = require("../../aws-utils/aws-ssm");
const resolve_appId_1 = require("../resolve-appId");
const exp_backoff_executor_1 = require("./exp-backoff-executor");
const get_ssm_sdk_parameters_1 = require("./get-ssm-sdk-parameters");
const deleteEnvironmentParametersFromService = async (context, envName) => {
    let appId;
    try {
        appId = (0, resolve_appId_1.resolveAppId)(context);
    }
    catch (_a) {
        amplify_prompts_1.printer.debug(`No AppId found when deleting parameters for environment ${envName}`);
        return;
    }
    const { client } = await aws_ssm_1.SSM.getInstance(context);
    const envKeysInParameterStore = await getAllEnvParametersFromParameterStore(appId, envName, client);
    await deleteParametersFromParameterStore(client, envKeysInParameterStore);
};
exports.deleteEnvironmentParametersFromService = deleteEnvironmentParametersFromService;
const deleteEnvironmentParametersForResources = async (context, envName, resources) => {
    if (resources.length === 0) {
        return;
    }
    let appId;
    try {
        appId = (0, resolve_appId_1.resolveAppId)(context);
    }
    catch (_a) {
        amplify_prompts_1.printer.debug(`No AppId found when deleting parameters for environment ${envName}`);
        return;
    }
    const { client } = await aws_ssm_1.SSM.getInstance(context);
    const envKeysInParameterStore = await getAllEnvParametersFromParameterStore(appId, envName, client);
    const removedParameterKeys = envKeysInParameterStore.filter((key) => {
        const keyAfterPaths = key.split('/').pop();
        const [, categoryName, resourceName] = keyAfterPaths.split('_');
        for (const resource of resources) {
            if (resource.category === categoryName && resource.resourceName === resourceName) {
                return true;
            }
        }
        return false;
    });
    await deleteParametersFromParameterStore(client, removedParameterKeys);
};
exports.deleteEnvironmentParametersForResources = deleteEnvironmentParametersForResources;
const deleteParametersFromParameterStore = async (ssmClient, parameterKeys) => {
    if (parameterKeys.length === 0) {
        return;
    }
    try {
        const chunkedKeys = chunkForParameterStore(parameterKeys);
        const deleteKeysFromPSPromises = chunkedKeys.map((keys) => {
            const ssmArgument = (0, get_ssm_sdk_parameters_1.getSsmSdkParametersDeleteParameters)(keys);
            return () => ssmClient.deleteParameters(ssmArgument).promise();
        });
        await (0, exp_backoff_executor_1.executeSdkPromisesWithExponentialBackOff)(deleteKeysFromPSPromises);
    }
    catch (e) {
        throw new amplify_cli_core_1.AmplifyFault('ParametersDeleteFault', {
            message: `Failed to delete parameters from the service`,
        }, e);
    }
};
function isAmplifyParameter(parameter) {
    const keyPrefix = 'AMPLIFY_';
    const splitParam = parameter.split('/');
    const lastPartOfPath = splitParam.slice(-1).pop();
    return lastPartOfPath.startsWith(keyPrefix);
}
const getAllEnvParametersFromParameterStore = async (appId, envName, ssmClient) => {
    const parametersUnderPath = [];
    let receivedNextToken = '';
    do {
        const ssmArgument = (0, get_ssm_sdk_parameters_1.getSsmSdkParametersGetParametersByPath)(appId, envName, receivedNextToken);
        const [data] = await (0, exp_backoff_executor_1.executeSdkPromisesWithExponentialBackOff)([() => ssmClient.getParametersByPath(ssmArgument).promise()]);
        parametersUnderPath.push(...data.Parameters.map((returnedParameter) => returnedParameter.Name).filter((name) => isAmplifyParameter(name)));
        receivedNextToken = data.NextToken;
    } while (receivedNextToken);
    return parametersUnderPath;
};
const chunkForParameterStore = (keys) => {
    const maxLength = 10;
    const chunkedKeys = [];
    let lastChunk = [];
    chunkedKeys.push(lastChunk);
    keys.forEach((key) => {
        if (lastChunk.length === maxLength) {
            lastChunk = [];
            chunkedKeys.push(lastChunk);
        }
        lastChunk.push(key);
    });
    return chunkedKeys;
};
//# sourceMappingURL=delete-ssm-parameters.js.map